@@ -27,11 +27,11 @@ except ImportError:
 
 from . import components
 from .components import DashboardComponent
-from .core import BokehServer, format_bytes
+from .core import BokehServer
 from .worker import SystemMonitor, format_time, counters_doc
 from .utils import transpose
 from ..metrics import time
-from ..utils import log_errors
+from ..utils import log_errors, format_bytes
 from ..diagnostics.progress_stream import color_of, progress_quads, nbytes_bar
 from ..diagnostics.progress import AllProgress
 from .task_stream import TaskStreamPlugin
@@ -53,6 +53,8 @@ with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
 
 template = jinja2.Template(template_source)
 
+template_variables = {'pages': ['status', 'workers', 'tasks', 'system', 'counters']}
+
 
 def update(source, data):
     """ Update source with data
@@ -629,7 +631,7 @@ class TaskProgress(DashboardComponent):
         y_range = Range1d(-8, 0)
 
         self.root = figure(
-            id='bk-task-progress-plot',
+            id='bk-task-progress-plot', title='Progress',
             x_range=x_range, y_range=y_range, toolbar_location=None, **kwargs
         )
         self.root.line(  # just to define early ranges
@@ -885,6 +887,8 @@ def systemmonitor_doc(scheduler, doc):
         doc.add_root(column(table.root, sysmon.root,
                             sizing_mode='scale_width'))
         doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'system'
 
 
 def stealing_doc(scheduler, doc):
@@ -905,6 +909,8 @@ def stealing_doc(scheduler, doc):
                             sizing_mode='scale_width'))
 
         doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'stealing'
 
 
 def events_doc(scheduler, doc):
@@ -915,6 +921,8 @@ def events_doc(scheduler, doc):
         doc.title = "Dask Scheduler Events"
         doc.add_root(column(events.root, sizing_mode='scale_width'))
         doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'events'
 
 
 def workers_doc(scheduler, doc):
@@ -925,6 +933,8 @@ def workers_doc(scheduler, doc):
         doc.title = "Dask Workers"
         doc.add_root(table.root)
         doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'workers'
 
 
 def tasks_doc(scheduler, doc):
@@ -936,6 +946,8 @@ def tasks_doc(scheduler, doc):
         doc.title = "Dask Task Stream"
         doc.add_root(ts.root)
         doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'tasks'
 
 
 def status_doc(scheduler, doc):
@@ -970,11 +982,15 @@ def status_doc(scheduler, doc):
                             task_progress.root,
                             sizing_mode='scale_width'))
         doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'status'
 
 
 class BokehScheduler(BokehServer):
-    def __init__(self, scheduler, io_loop=None):
+    def __init__(self, scheduler, io_loop=None, **kwargs):
         self.scheduler = scheduler
+        self.server_kwargs = kwargs
+
         systemmonitor = Application(FunctionHandler(partial(systemmonitor_doc,
                                                             scheduler)))
         workers = Application(FunctionHandler(partial(workers_doc, scheduler)))
