@@ -3,6 +3,7 @@ from __future__ import print_function, division, absolute_import
 from functools import partial
 import logging
 import math
+import os
 
 from bokeh.layouts import row, column, widgetbox
 from bokeh.models import ( ColumnDataSource, DataRange1d, HoverTool,
@@ -15,16 +16,24 @@ from bokeh.palettes import RdBu
 from toolz import merge, partition_all
 
 from .components import DashboardComponent
-from .core import BokehServer, format_bytes, format_time
+from .core import BokehServer, format_time
 from .utils import transpose
 from ..compatibility import WINDOWS
 from ..diagnostics.progress_stream import color_of
 from ..metrics import time
-from ..utils import log_errors, key_split
+from ..utils import log_errors, key_split, format_bytes
 
 
 logger = logging.getLogger(__name__)
 
+import jinja2
+
+with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
+    template_source = f.read()
+
+template = jinja2.Template(template_source)
+template_variables = {'pages': ['main', 'system', 'crossfilter', 'counters']}
+
 
 class StateTable(DashboardComponent):
     """ Currently running tasks """
@@ -343,7 +352,9 @@ class SystemMonitor(DashboardComponent):
         self.worker = worker
 
         names = worker.monitor.quantities
+        self.last = 0
         self.source = ColumnDataSource({name: [] for name in names})
+        self.source.data.update(self.get_data())
 
         x_range = DataRange1d(follow='end', follow_interval=20000,
                               range_padding=0)
@@ -392,16 +403,18 @@ class SystemMonitor(DashboardComponent):
         self.cpu.y_range.start = 0
         self.bandwidth.y_range.start = 0
 
-        self.last = 0
         self.root = column(*plots, **kw)
         self.worker.monitor.update()
 
+    def get_data(self):
+        d = self.worker.monitor.range_query(start=self.last)
+        d['time'] = [x * 1000 for x in d['time']]
+        self.last = self.worker.monitor.count
+        return d
+
     def update(self):
         with log_errors():
-            d = self.worker.monitor.range_query(start=self.last)
-            d['time'] = [x * 1000 for x in d['time']]
-            self.source.stream(d, 1000)
-            self.last = self.worker.monitor.count
+            self.source.stream(self.get_data(), 1000)
 
 
 class Counters(DashboardComponent):
@@ -460,7 +473,8 @@ class Counters(DashboardComponent):
     def add_counter_figure(self, name):
         with log_errors():
             n = len(self.server.counters[name].intervals)
-            sources = {i: ColumnDataSource({'x': [], 'y': [], 'y-center': []})
+            sources = {i: ColumnDataSource({'x': [], 'y': [],
+                                            'y-center': [], 'counts': []})
                         for i in range(n)}
 
             fig = figure(title=name, tools='', height=150,
@@ -493,6 +507,7 @@ class Counters(DashboardComponent):
                 for i, d in enumerate(digest.components):
                     if d.size():
                         ys, xs = d.histogram(100)
+                        xs = xs[1:]
                         if name.endswith('duration'):
                             xs *= 1000
                         self.digest_sources[name][i].data.update({'x': xs, 'y': ys})
@@ -509,9 +524,9 @@ class Counters(DashboardComponent):
                         ys = [factor * c for c in counts]
                         y_centers = [y / 2 for y in ys]
                         xs = list(map(str, xs))
-                        self.counter_sources[name][i].data.update(
-                                {'x': xs, 'y': ys, 'y-center': y_centers,
-                                 'counts': counts})
+                        d = {'x': xs, 'y': ys, 'y-center': y_centers,
+                             'counts': counts}
+                        self.counter_sources[name][i].data.update(d)
                     fig.title.text = '%s: %d' % (name, counter.size())
                     fig.x_range.factors = list(map(str, xs))
 
@@ -543,6 +558,9 @@ def main_doc(worker, doc):
                             communicating_ts.root,
                             communicating_stream.root,
                             sizing_mode='scale_width'))
+        doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'main'
 
 
 def crossfilter_doc(worker, doc):
@@ -555,6 +573,9 @@ def crossfilter_doc(worker, doc):
         doc.add_periodic_callback(crossfilter.update, 500)
 
         doc.add_root(column(statetable.root, crossfilter.root))
+        doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'crossfilter'
 
 
 def systemmonitor_doc(worker, doc):
@@ -564,6 +585,9 @@ def systemmonitor_doc(worker, doc):
         doc.add_periodic_callback(sysmon.update, 500)
 
         doc.add_root(sysmon.root)
+        doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'system'
 
 
 def counters_doc(server, doc):
@@ -573,11 +597,15 @@ def counters_doc(server, doc):
         doc.add_periodic_callback(counter.update, 500)
 
         doc.add_root(counter.root)
+        doc.template = template
+        doc.template_variables.update(template_variables)
+        doc.template_variables['active_page'] = 'counters'
 
 
 class BokehWorker(BokehServer):
-    def __init__(self, worker, io_loop=None):
+    def __init__(self, worker, io_loop=None, **kwargs):
         self.worker = worker
+        self.server_kwargs = kwargs
         main = Application(FunctionHandler(partial(main_doc, worker)))
         crossfilter = Application(FunctionHandler(partial(crossfilter_doc, worker)))
         systemmonitor = Application(FunctionHandler(partial(systemmonitor_doc, worker)))
