@@ -2,10 +2,12 @@ from __future__ import print_function, division, absolute_import
 
 import atexit
 from datetime import timedelta
+from functools import partial
 import json
 import logging
 import os
 import shutil
+import signal
 from sys import exit
 from time import sleep
 
@@ -16,7 +18,8 @@ from distributed.utils import All, get_ip_interface
 from distributed.worker import _ncores
 from distributed.http import HTTPWorker
 from distributed.metrics import time
-from distributed.cli.utils import check_python_3
+from distributed.security import Security
+from distributed.cli.utils import check_python_3, uri_from_host_port
 
 from toolz import valmap
 from tornado.ioloop import IOLoop, TimeoutError
@@ -26,7 +29,6 @@ logger = logging.getLogger('distributed.dask_worker')
 
 global_nannies = []
 
-import signal
 
 def handle_signal(sig, frame):
     loop = IOLoop.instance()
@@ -38,11 +40,20 @@ def handle_signal(sig, frame):
     if loop._running:
         loop.add_callback_from_signal(loop.stop)
     else:
+        loop.close()
         exit(1)
 
 
+pem_file_option_type = click.Path(exists=True, resolve_path=True)
+
 @click.command()
 @click.argument('scheduler', type=str, required=False)
+@click.option('--tls-ca-file', type=pem_file_option_type, default=None,
+              help="CA cert(s) file for TLS (in PEM format)")
+@click.option('--tls-cert', type=pem_file_option_type, default=None,
+              help="certificate file for TLS (in PEM format)")
+@click.option('--tls-key', type=pem_file_option_type, default=None,
+              help="private key file for TLS (in PEM format)")
 @click.option('--worker-port', type=int, default=0,
               help="Serving computation port, defaults to random")
 @click.option('--http-port', type=int, default=0,
@@ -78,8 +89,6 @@ def handle_signal(sig, frame):
               help="File to write the process PID")
 @click.option('--local-directory', default='', type=str,
               help="Directory to place worker files")
-@click.option('--temp-filename', default=None,
-              help="Internal use only")
 @click.option('--resources', type=str, default='',
               help='Resources for task constraints like "GPU=2 MEM=10e9"')
 @click.option('--scheduler-file', type=str, default='',
@@ -87,13 +96,21 @@ def handle_signal(sig, frame):
                    'Use with dask-scheduler --scheduler-file')
 @click.option('--death-timeout', type=float, default=None,
               help="Seconds to wait for a scheduler before closing")
+@click.option('--bokeh-prefix', type=str, default=None,
+              help="Prefix for the bokeh app")
 @click.option('--preload', type=str, multiple=True,
               help='Module that should be loaded by each worker process '
                    'like "foo.bar" or "/path/to/foo.py"')
 def main(scheduler, host, worker_port, http_port, nanny_port, nthreads, nprocs,
-         nanny, name, memory_limit, pid_file, temp_filename, reconnect,
+         nanny, name, memory_limit, pid_file, reconnect,
          resources, bokeh, bokeh_port, local_directory, scheduler_file,
-         interface, death_timeout, preload):
+         interface, death_timeout, preload, bokeh_prefix,
+         tls_ca_file, tls_cert, tls_key):
+    sec = Security(tls_ca_file=tls_ca_file,
+                   tls_worker_cert=tls_cert,
+                   tls_worker_key=tls_key,
+                   )
+
     if nanny:
         port = nanny_port
     else:
@@ -127,7 +144,11 @@ def main(scheduler, host, worker_port, http_port, nanny_port, nthreads, nprocs,
         except ImportError:
             pass
         else:
-            services[('bokeh', bokeh_port)] = BokehWorker
+            if bokeh_prefix:
+                result = (BokehWorker, {'prefix': bokeh_prefix})
+            else:
+                result = BokehWorker
+            services[('bokeh', bokeh_port)] = result
 
     if resources:
         resources = resources.replace(',', ' ').split()
@@ -167,7 +188,7 @@ def main(scheduler, host, worker_port, http_port, nanny_port, nthreads, nprocs,
                  services=services, name=name, loop=loop, resources=resources,
                  memory_limit=memory_limit, reconnect=reconnect,
                  local_dir=local_directory, death_timeout=death_timeout,
-                 preload=preload,
+                 preload=preload, security=sec,
                  **kwargs)
                for i in range(nprocs)]
 
@@ -177,26 +198,17 @@ def main(scheduler, host, worker_port, http_port, nanny_port, nthreads, nprocs,
         else:
             host = get_ip_interface(interface)
 
+    if host or port:
+        addr = uri_from_host_port(host, port, 0)
+    else:
+        # Choose appropriate address for scheduler
+        addr = None
+
     for n in nannies:
-        if host:
-            n.start((host, port))
-        else:
-            n.start(port)
+        n.start(addr)
         if t is Nanny:
             global_nannies.append(n)
 
-    if temp_filename:
-        @gen.coroutine
-        def f():
-            while nannies[0].status != 'running':
-                yield gen.sleep(0.01)
-            import json
-            msg = {'port': nannies[0].port,
-                   'local_directory': nannies[0].local_dir}
-            with open(temp_filename, 'w') as f:
-                json.dump(msg, f)
-        loop.add_callback(f)
-
     @gen.coroutine
     def run():
         while all(n.status != 'closed' for n in nannies):
@@ -216,15 +228,17 @@ def main(scheduler, host, worker_port, http_port, nanny_port, nthreads, nprocs,
 
     @gen.coroutine
     def f():
-        with rpc(nannies[0].scheduler.address) as scheduler:
-            if nanny:
+        if nanny:
+            w = nannies[0]
+            with w.rpc(w.scheduler.address) as scheduler:
                 yield gen.with_timeout(
                         timeout=timedelta(seconds=2),
                         future=All([scheduler.unregister(address=n.worker_address, close=True)
-                                   for n in nannies if n.process and n.worker_address]),
+                                    for n in nannies if n.process and n.worker_address]),
                         io_loop=loop2)
 
     loop2.run_sync(f)
+    loop2.close()
 
     if nanny:
         for n in nannies:
