@@ -1,20 +1,21 @@
 from __future__ import print_function, division, absolute_import
 
-from datetime import timedelta
 import errno
 import logging
 import socket
+import ssl
 import struct
 import sys
 
-from tornado import gen
-from tornado.iostream import IOStream, StreamClosedError
+import tornado
+from tornado import gen, netutil
+from tornado.iostream import StreamClosedError
 from tornado.tcpclient import TCPClient
 from tornado.tcpserver import TCPServer
 
 from .. import config
 from ..compatibility import finalize
-from ..utils import ensure_bytes, ensure_ip, get_ip, get_ipv6
+from ..utils import ensure_bytes, ensure_ip, get_ip, get_ipv6, nbytes
 
 from .registry import Backend, backends
 from .addressing import parse_host_port, unparse_host_port
@@ -90,7 +91,7 @@ def set_tcp_timeout(stream):
             TCP_USER_TIMEOUT = 18  # since Linux 2.6.37
             sock.setsockopt(socket.SOL_TCP, TCP_USER_TIMEOUT, timeout * 1000)
     except EnvironmentError as e:
-        logger.warn("Could not set timeout on TCP stream: %s", e)
+        logger.warning("Could not set timeout on TCP stream: %s", e)
 
 
 def convert_stream_closed_error(exc):
@@ -109,6 +110,7 @@ class TCP(Comm):
     """
     An established communication based on an underlying Tornado IOStream.
     """
+    _iostream_allows_memoryview = tornado.version_info >= (4, 5)
 
     def __init__(self, stream, peer_addr, deserialize=True):
         self._peer_addr = peer_addr
@@ -116,20 +118,25 @@ class TCP(Comm):
         self.deserialize = deserialize
         self._finalizer = finalize(self, self._get_finalizer())
         self._finalizer.atexit = False
+        self._extra = {}
 
         stream.set_nodelay(True)
         set_tcp_timeout(stream)
+        self._read_extra()
+
+    def _read_extra(self):
+        pass
 
     def _get_finalizer(self):
         def finalize(stream=self.stream, r=repr(self)):
             if not stream.closed():
-                logger.warn("Closing dangling stream in %s" % (r,))
+                logger.warning("Closing dangling stream in %s" % (r,))
                 stream.close()
 
         return finalize
 
     def __repr__(self):
-        return "<TCP %r>" % (self._peer_addr,)
+        return "<%s %r>" % (self.__class__.__name__, self._peer_addr,)
 
     @property
     def peer_address(self):
@@ -158,7 +165,12 @@ class TCP(Comm):
             self.stream = None
             convert_stream_closed_error(e)
 
-        msg = from_frames(frames, deserialize=self.deserialize)
+        try:
+            msg = from_frames(frames, deserialize=self.deserialize)
+        except EOFError:
+            # Frames possibly garbled or truncated by communication error
+            self.abort()
+            raise CommClosedError("aborted stream on truncated data")
         raise gen.Return(msg)
 
     @gen.coroutine
@@ -167,12 +179,14 @@ class TCP(Comm):
         if stream is None:
             raise CommClosedError
 
-        # IOStream.write() only takes bytes objects, not memoryviews
-        frames = [ensure_bytes(f) for f in to_frames(msg)]
+        if self._iostream_allows_memoryview:
+            frames = to_frames(msg)
+        else:
+            frames = [ensure_bytes(f) for f in to_frames(msg)]
 
         try:
             lengths = ([struct.pack('Q', len(frames))] +
-                       [struct.pack('Q', len(frame)) for frame in frames])
+                       [struct.pack('Q', nbytes(frame)) for frame in frames])
             stream.write(b''.join(lengths))
 
             for frame in frames:
@@ -210,39 +224,111 @@ class TCP(Comm):
     def closed(self):
         return self.stream is None or self.stream.closed()
 
+    @property
+    def extra_info(self):
+        return self._extra
 
-class TCPConnector(object):
+
+class TLS(TCP):
+    """
+    A TLS-specific version of TCP.
+    """
+
+    def _read_extra(self):
+        TCP._read_extra(self)
+        sock = self.stream.socket
+        if sock is not None:
+            self._extra.update(peercert=sock.getpeercert(),
+                               cipher=sock.cipher())
+            cipher, proto, bits = self._extra['cipher']
+            logger.debug("TLS connection with %r: protocol=%s, cipher=%s, bits=%d",
+                         self._peer_addr, proto, cipher, bits)
+
+
+def _expect_tls_context(connection_args):
+    ctx = connection_args.get('ssl_context')
+    if not isinstance(ctx, ssl.SSLContext):
+        raise TypeError("TLS expects a `ssl_context` argument of type "
+                        "ssl.SSLContext (perhaps check your TLS configuration?)")
+    return ctx
+
+
+class RequireEncryptionMixin(object):
+
+    def _check_encryption(self, address, connection_args):
+        if not self.encrypted and connection_args.get('require_encryption'):
+            # XXX Should we have a dedicated SecurityError class?
+            raise RuntimeError("encryption required by Dask configuration, "
+                               "refusing communication from/to %r"
+                               % (self.prefix + address,))
+
+
+class BaseTCPConnector(Connector, RequireEncryptionMixin):
 
     @gen.coroutine
-    def connect(self, address, deserialize=True):
+    def connect(self, address, deserialize=True, **connection_args):
+        self._check_encryption(address, connection_args)
         ip, port = parse_host_port(address)
+        kwargs = self._get_connect_args(**connection_args)
 
         client = TCPClient()
         try:
             stream = yield client.connect(ip, port,
-                                          max_buffer_size=MAX_BUFFER_SIZE)
+                                          max_buffer_size=MAX_BUFFER_SIZE,
+                                          **kwargs)
         except StreamClosedError as e:
             # The socket connect() call failed
             convert_stream_closed_error(e)
 
-        raise gen.Return(TCP(stream, 'tcp://' + address, deserialize))
+        # XXX
+        raise gen.Return(self.comm_class(stream,
+                                         self.prefix + address,
+                                         deserialize))
 
 
-class TCPListener(Listener):
+class TCPConnector(BaseTCPConnector):
+    prefix = 'tcp://'
+    comm_class = TCP
+    encrypted = False
 
-    def __init__(self, address, comm_handler, deserialize=True, default_port=0):
+    def _get_connect_args(self, **connection_args):
+        return {}
+
+
+class TLSConnector(BaseTCPConnector):
+    prefix = 'tls://'
+    comm_class = TLS
+    encrypted = True
+
+    def _get_connect_args(self, **connection_args):
+        ctx = _expect_tls_context(connection_args)
+        return {'ssl_options': ctx}
+
+
+class BaseTCPListener(Listener, RequireEncryptionMixin):
+
+    def __init__(self, address, comm_handler, deserialize=True,
+                 default_port=0, **connection_args):
+        self._check_encryption(address, connection_args)
         self.ip, self.port = parse_host_port(address, default_port)
         self.comm_handler = comm_handler
         self.deserialize = deserialize
+        self.server_args = self._get_server_args(**connection_args)
         self.tcp_server = None
         self.bound_address = None
 
     def start(self):
-        self.tcp_server = TCPServer(max_buffer_size=MAX_BUFFER_SIZE)
+        self.tcp_server = TCPServer(max_buffer_size=MAX_BUFFER_SIZE,
+                                    **self.server_args)
         self.tcp_server.handle_stream = self.handle_stream
         for i in range(5):
             try:
-                self.tcp_server.listen(self.port, self.ip)
+                # When shuffling data between workers, there can
+                # really be O(cluster size) connection requests
+                # on a single worker socket, make sure the backlog
+                # is large enough not to lose any.
+                sockets = netutil.bind_sockets(self.port, address=self.ip,
+                                               backlog=2048)
             except EnvironmentError as e:
                 # EADDRINUSE can happen sporadically when trying to bind
                 # to an ephemeral port
@@ -250,6 +336,7 @@ class TCPListener(Listener):
                     raise
                 exc = e
             else:
+                self.tcp_server.add_sockets(sockets)
                 break
         else:
             raise exc
@@ -279,7 +366,7 @@ class TCPListener(Listener):
         """
         The listening address as a string.
         """
-        return 'tcp://' + unparse_host_port(*self.get_host_port())
+        return self.prefix + unparse_host_port(*self.get_host_port())
 
     @property
     def contact_address(self):
@@ -288,23 +375,54 @@ class TCPListener(Listener):
         """
         host, port = self.get_host_port()
         host = ensure_concrete_host(host)
-        return 'tcp://' + unparse_host_port(host, port)
+        return self.prefix + unparse_host_port(host, port)
+
+
+class TCPListener(BaseTCPListener):
+    prefix = 'tcp://'
+    encrypted = False
+
+    def _get_server_args(self, **connection_args):
+        return {}
 
     def handle_stream(self, stream, address):
-        address = 'tcp://' + unparse_host_port(*address[:2])
+        address = self.prefix + unparse_host_port(*address[:2])
         comm = TCP(stream, address, self.deserialize)
         self.comm_handler(comm)
 
 
-class TCPBackend(Backend):
+class TLSListener(BaseTCPListener):
+    prefix = 'tls://'
+    encrypted = True
+
+    def _get_server_args(self, **connection_args):
+        ctx = _expect_tls_context(connection_args)
+        return {'ssl_options': ctx}
+
+    @gen.coroutine
+    def handle_stream(self, stream, address):
+        address = self.prefix + unparse_host_port(*address[:2])
+        try:
+            yield stream.wait_for_handshake()
+        except EnvironmentError as e:
+            # The handshake went wrong, log and ignore
+            logger.warning("listener on %r: TLS handshake failed with remote %r: %s",
+                           self.listen_address, address,
+                           getattr(e, "real_error", None) or e)
+        else:
+            comm = TLS(stream, address, self.deserialize)
+            self.comm_handler(comm)
+
+
+class BaseTCPBackend(Backend):
 
     # I/O
 
     def get_connector(self):
-        return TCPConnector()
+        return self._connector_class()
 
-    def get_listener(self, loc, handle_comm, deserialize):
-        return TCPListener(loc, handle_comm, deserialize)
+    def get_listener(self, loc, handle_comm, deserialize, **connection_args):
+        return self._listener_class(loc, handle_comm, deserialize, **connection_args)
 
     # Address handling
 
@@ -328,4 +446,15 @@ class TCPBackend(Backend):
         return unparse_host_port(local_host, None)
 
 
+class TCPBackend(BaseTCPBackend):
+    _connector_class = TCPConnector
+    _listener_class = TCPListener
+
+
+class TLSBackend(BaseTCPBackend):
+    _connector_class = TLSConnector
+    _listener_class = TLSListener
+
+
 backends['tcp'] = TCPBackend()
+backends['tls'] = TLSBackend()
