@@ -88,8 +88,6 @@ class Server(object):
         self._listen_address = None
         self._port = None
         self._comms = {}
-        self.rpc = ConnectionPool(limit=connection_limit,
-                                  deserialize=deserialize)
         self.deserialize = deserialize
         self.monitor = SystemMonitor()
         self.counters = None
@@ -124,7 +122,13 @@ class Server(object):
         if not self.__stopped:
             self.__stopped = True
             if self.listener is not None:
-                self.listener.stop()
+                # Delay closing the server socket until the next IO loop tick.
+                # Otherwise race conditions can appear if an event handler
+                # for an accept() call is already scheduled by the IO loop,
+                # raising EBADF.
+                # The demonstrator for this is Worker.terminate(), which
+                # closes the server socket in response to an incoming message.
+                self.io_loop.add_callback(self.listener.stop)
 
     def _measure_tick(self):
         now = time()
@@ -179,7 +183,7 @@ class Server(object):
     def identity(self, comm):
         return {'type': type(self).__name__, 'id': self.id}
 
-    def listen(self, port_or_addr=None):
+    def listen(self, port_or_addr=None, listen_args=None):
         if port_or_addr is None:
             port_or_addr = self.default_port
         if isinstance(port_or_addr, int):
@@ -190,7 +194,8 @@ class Server(object):
             addr = port_or_addr
             assert isinstance(addr, string_types)
         self.listener = listen(addr, self.handle_comm,
-                               deserialize=self.deserialize)
+                               deserialize=self.deserialize,
+                               connection_args=listen_args)
         self.listener.start()
 
     @gen.coroutine
@@ -205,6 +210,9 @@ class Server(object):
 
         Coroutines should expect a single Comm object.
         """
+        if self.__stopped:
+            comm.abort()
+            return
         address = comm.peer_address
         op = None
 
@@ -242,7 +250,7 @@ class Server(object):
                     handler = self.handlers[op]
                 except KeyError:
                     result = "No handler found: %s" % op
-                    logger.warn(result, exc_info=True)
+                    logger.warning(result, exc_info=True)
                 else:
                     logger.debug("Calling into handler %s", handler.__name__)
                     try:
@@ -250,7 +258,7 @@ class Server(object):
                         if type(result) is gen.Future:
                             result = yield result
                     except CommClosedError as e:
-                        logger.warn("Lost connection to %r: %s", address, e)
+                        logger.warning("Lost connection to %r: %s", address, e)
                         break
                     except Exception as e:
                         logger.exception(e)
@@ -259,8 +267,8 @@ class Server(object):
                     try:
                         yield comm.write(result)
                     except EnvironmentError as e:
-                        logger.warn("Lost connection to %r while sending result for op %r: %s",
-                                    address, op, e)
+                        logger.warning("Lost connection to %r while sending result for op %r: %s",
+                                       address, op, e)
                         break
                 msg = result = None
                 if close_desired:
@@ -283,24 +291,17 @@ def pingpong(comm):
 
 
 @gen.coroutine
-def send_recv(comm=None, addr=None, reply=True, deserialize=True, **kwargs):
+def send_recv(comm, reply=True, deserialize=True, **kwargs):
     """ Send and recv with a Comm.
 
     Keyword arguments turn into the message
 
     response = yield send_recv(comm, op='ping', reply=True)
     """
-    assert (comm is None) != (addr is None)
-
     msg = kwargs
     msg['reply'] = reply
     please_close = kwargs.get('close')
-
-    if comm is None:
-        comm = yield connect(addr_from_args(addr), deserialize=deserialize)
-        force_close = True
-    else:
-        force_close = False
+    force_close = False
 
     try:
         yield comm.write(msg)
@@ -351,12 +352,14 @@ class rpc(object):
     comms = ()
     address = None
 
-    def __init__(self, arg=None, comm=None, deserialize=True, timeout=3):
+    def __init__(self, arg=None, comm=None, deserialize=True, timeout=3,
+                 connection_args=None):
         self.comms = {}
         self.address = coerce_to_address(arg)
         self.timeout = timeout
         self.status = 'running'
         self.deserialize = deserialize
+        self.connection_args = connection_args
         rpc.active += 1
 
     @gen.coroutine
@@ -391,7 +394,8 @@ class rpc(object):
             del self.comms[s]
         if not open or comm.closed():
             comm = yield connect(self.address, self.timeout,
-                                 deserialize=self.deserialize)
+                                 deserialize=self.deserialize,
+                                 connection_args=self.connection_args)
         self.comms[comm] = False     # mark as taken
         raise gen.Return(comm)
 
@@ -443,8 +447,8 @@ class rpc(object):
             self.status = 'closed'
             still_open = [comm for comm in self.comms if not comm.closed()]
             if still_open:
-                logger.warn("rpc object %s deleted with %d open comms",
-                            self, len(still_open))
+                logger.warning("rpc object %s deleted with %d open comms",
+                               self, len(still_open))
                 for comm in still_open:
                     comm.abort()
 
@@ -477,6 +481,13 @@ class PooledRPCCall(object):
     def close_rpc(self):
         pass
 
+    # For compatibility with rpc()
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        pass
+
     def __repr__(self):
         return "<pooled rpc to %r>" % (self.addr,)
 
@@ -514,7 +525,7 @@ class ConnectionPool(object):
     deserialize: bool
         Whether or not to deserialize data by default or pass it through
     """
-    def __init__(self, limit=512, deserialize=True):
+    def __init__(self, limit=512, deserialize=True, connection_args=None):
         self.open = 0          # Total number of open comms
         self.active = 0        # Number of comms currently in use
         self.limit = limit     # Max number of open comms
@@ -523,6 +534,7 @@ class ConnectionPool(object):
         # Invariant: len(occupied) == active
         self.occupied = defaultdict(set)
         self.deserialize = deserialize
+        self.connection_args = connection_args
         self.event = Event()
 
     def __str__(self):
@@ -560,7 +572,8 @@ class ConnectionPool(object):
         self.open += 1
         try:
             comm = yield connect(addr, timeout=timeout,
-                                 deserialize=self.deserialize)
+                                 deserialize=self.deserialize,
+                                 connection_args=self.connection_args)
         except Exception:
             self.open -= 1
             raise
