@@ -12,7 +12,7 @@ from tornado import gen
 from tornado.ioloop import IOLoop
 
 from ..core import CommClosedError
-from ..utils import sync, ignoring, All
+from ..utils import sync, ignoring, All, silence_logging
 from ..nanny import Nanny
 from ..scheduler import Scheduler
 from ..worker import Worker, _ncores
@@ -63,17 +63,13 @@ class LocalCluster(object):
                  silence_logs=logging.CRITICAL, diagnostics_port=8787,
                  services={}, worker_services={}, nanny=None, **worker_kwargs):
         if nanny is not None:
-            warnings.warn("nanny has been deprecated, used processes=")
+            warnings.warning("nanny has been deprecated, used processes=")
             processes = nanny
         self.status = None
         self.processes = processes
         self.silence_logs = silence_logs
         if silence_logs:
-            for l in ['distributed.scheduler',
-                      'distributed.worker',
-                      'distributed.core',
-                      'distributed.nanny']:
-                logging.getLogger(l).setLevel(silence_logs)
+            silence_logging(level=silence_logs)
         if n_workers is None and threads_per_worker is None:
             if processes:
                 n_workers = _ncores
@@ -98,10 +94,12 @@ class LocalCluster(object):
         if diagnostics_port is not None:
             try:
                 from distributed.bokeh.scheduler import BokehScheduler
+                from distributed.bokeh.worker import BokehWorker
             except ImportError:
                 logger.debug("To start diagnostics web server please install Bokeh")
             else:
                 services[('bokeh', diagnostics_port)] = BokehScheduler
+                worker_services[('bokeh', 0)] = BokehWorker
 
         self.scheduler = Scheduler(loop=self.loop,
                                    services=services)
@@ -168,15 +166,6 @@ class LocalCluster(object):
         else:
             W = Worker
 
-        try:
-            from distributed.bokeh.worker import BokehWorker
-        except ImportError:
-            pass
-        else:
-            if 'services' not in kwargs:
-                kwargs['services'] = {}
-            kwargs['services'][('bokeh', 0)] = BokehWorker
-
         w = W(self.scheduler.address, loop=self.loop,
               death_timeout=death_timeout,
               silence_logs=self.silence_logs, **kwargs)
