@@ -67,8 +67,7 @@ def test_processing(c, s, a, b):
 
 @gen_cluster()
 def test_services(s, a, b):
-    c = Worker(s.ip, s.port, ncores=1, ip='127.0.0.1',
-               services={'http': HTTPWorker})
+    c = Worker(s.ip, s.port, ncores=1, services={'http': HTTPWorker})
     yield c._start()
     assert isinstance(c.services['http'], HTTPServer)
     assert c.service_ports['http'] == c.services['http'].port
@@ -79,8 +78,7 @@ def test_services(s, a, b):
 
 @gen_cluster()
 def test_services_port(s, a, b):
-    c = Worker(s.ip, s.port, ncores=1, ip='127.0.0.1',
-               services={('http', 9898): HTTPWorker})
+    c = Worker(s.ip, s.port, ncores=1, services={('http', 9898): HTTPWorker})
     yield c._start()
     assert isinstance(c.services['http'], HTTPServer)
     assert (c.service_ports['http']
