@@ -77,7 +77,7 @@ def test_recompute_released_results(c, s, a, b):
 
     z = delayed(dec)(x)
     zz = c.compute(z)
-    result = yield zz._result()
+    result = yield zz
     assert result == 1
 
 
@@ -131,7 +131,7 @@ def test_no_valid_workers(client, s, a, b, c):
     assert x.key in s.unrunnable
 
     with pytest.raises(gen.TimeoutError):
-        yield gen.with_timeout(timedelta(milliseconds=50), x._result())
+        yield gen.with_timeout(timedelta(milliseconds=50), x)
 
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 3)
@@ -139,7 +139,7 @@ def test_no_valid_workers_loose_restrictions(client, s, a, b, c):
     x = client.submit(inc, 1, workers='127.0.0.5:9999',
                       allow_other_workers=True)
 
-    result = yield x._result()
+    result = yield x
     assert result == 2
 
 
@@ -152,7 +152,12 @@ def test_no_workers(client, s):
     assert x.key in s.unrunnable
 
     with pytest.raises(gen.TimeoutError):
-        yield gen.with_timeout(timedelta(milliseconds=50), x._result())
+        yield gen.with_timeout(timedelta(milliseconds=50), x)
+
+
+@gen_cluster(ncores=[])
+def test_retire_workers_empty(s):
+    yield s.retire_workers(workers=[])
 
 
 @pytest.mark.skip
@@ -426,7 +431,7 @@ def test_delete_data(c, s, a, b):
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)])
 def test_delete(c, s, a):
     x = c.submit(inc, 1)
-    yield x._result()
+    yield x
     assert x.key in a.data
 
     yield c._cancel(x)
@@ -873,7 +878,14 @@ def test_include_communication_in_occupancy(c, s, a, b):
     while z.key not in s.rprocessing:
         yield gen.sleep(0.01)
 
-    assert s.processing[b.address][z.key] > 1
+    try:
+        assert s.processing[b.address][z.key] > 1
+    except Exception:
+        print("processing:", s.processing)
+        print("rprocessing:", s.rprocessing)
+        print("task_duration:", s.task_duration)
+        print("nbytes:", s.nbytes)
+        raise
 
 
 @gen_cluster(client=True)
@@ -1081,8 +1093,7 @@ def test_scheduler_file():
             data = json.load(f)
         assert data['address'] == s.address
 
-        c = Client(scheduler_file=fn, loop=s.loop, start=False)
-        yield c._start()
+        c = yield Client(scheduler_file=fn, loop=s.loop, asynchronous=True)
     yield s.close()
 
 
@@ -1094,3 +1105,18 @@ def test_non_existent_worker(c, s):
     yield gen.sleep(4)
     assert not s.workers
     assert all(v == 'no-worker' for v in s.task_state.values())
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 3)
+def test_correct_bad_time_estimate(c, s, *workers):
+    future = c.submit(slowinc, 1, delay=0)
+    yield _wait(future)
+
+    futures = [c.submit(slowinc, future, delay=0.1, pure=False)
+               for i in range(20)]
+
+    yield gen.sleep(0.5)
+
+    yield _wait(futures)
+
+    assert all(w.data for w in workers)
