@@ -20,7 +20,7 @@ import tempfile
 import threading
 import warnings
 
-from .compatibility import cache_from_source, invalidate_caches, reload
+from .compatibility import cache_from_source, getargspec, invalidate_caches, reload
 
 try:
     import resource
@@ -72,7 +72,7 @@ def has_arg(func, argname):
     """
     while True:
         try:
-            if argname in inspect.getargspec(func).args:
+            if argname in getargspec(func).args:
                 return True
         except TypeError:
             break
@@ -377,6 +377,21 @@ def log_errors(pdb=False):
         raise
 
 
+def silence_logging(level, root='distributed'):
+    """
+    Force all existing loggers below *root* to the given level at least
+    (or keep the existing level if less verbose).
+    """
+    if isinstance(level, str):
+        level = logging_names[level.upper()]
+
+    for name, logger in logging.root.manager.loggerDict.items():
+        if (isinstance(logger, logging.Logger)
+            and logger.name.startswith(root + '.')
+            and logger.level < level):
+            logger.setLevel(level)
+
+
 @memoize
 def ensure_ip(hostname):
     """ Ensure that address is an IP address
@@ -432,12 +447,23 @@ def truncate_exception(e, n=10000):
         return e
 
 
-def queue_to_iterator(q):
-    while True:
-        result = q.get()
-        if isinstance(result, StopIteration):
-            raise result
-        yield result
+if sys.version_info >= (3,):
+    # (re-)raising StopIteration is deprecated in 3.6+
+    exec("""def queue_to_iterator(q):
+        while True:
+            result = q.get()
+            if isinstance(result, StopIteration):
+                return result.value
+            yield result
+        """)
+else:
+    # Returning non-None from generator is a syntax error in 2.x
+    def queue_to_iterator(q):
+        while True:
+            result = q.get()
+            if isinstance(result, StopIteration):
+                raise result
+            yield result
 
 
 def _dump_to_queue(seq, q):
@@ -765,3 +791,40 @@ class itemgetter(object):
 
     def __reduce__(self):
         return (itemgetter, (self.index,))
+
+
+def format_bytes(n):
+    """ Format bytes as text
+
+    >>> format_bytes(1)
+    '1 B'
+    >>> format_bytes(1234)
+    '1.23 kB'
+    >>> format_bytes(12345678)
+    '12.35 MB'
+    >>> format_bytes(1234567890)
+    '1.23 GB'
+    """
+    if n > 1e9:
+        return '%0.2f GB' % (n / 1e9)
+    if n > 1e6:
+        return '%0.2f MB' % (n / 1e6)
+    if n > 1e3:
+        return '%0.2f kB' % (n / 1000)
+    return '%d B' % n
+
+
+if PY2:
+    def nbytes(frame):
+        """ Number of bytes of a frame or memoryview """
+        if type(frame) in (bytes, buffer):
+            return len(frame)
+        else:
+            return frame.nbytes
+else:
+    def nbytes(frame):
+        """ Number of bytes of a frame or memoryview """
+        if type(frame) is bytes:
+            return len(frame)
+        else:
+            return frame.nbytes
