@@ -4,6 +4,7 @@ from contextlib import contextmanager
 from datetime import timedelta
 import gc
 from glob import glob
+import inspect
 import logging
 import os
 import shutil
@@ -11,6 +12,7 @@ import signal
 import socket
 import subprocess
 import sys
+import tempfile
 import textwrap
 from time import sleep
 import uuid
@@ -22,9 +24,13 @@ from tornado import gen, queues
 from tornado.gen import TimeoutError
 from tornado.ioloop import IOLoop
 
+from .config import config
 from .core import connect, rpc, CommClosedError
 from .metrics import time
+from .nanny import Nanny
+from .security import Security
 from .utils import ignoring, log_errors, sync, mp_context, get_ip, get_ipv6
+from .worker import Worker
 import pytest
 
 
@@ -448,16 +454,17 @@ from .client import Client
 
 
 @gen.coroutine
-def start_cluster(ncores, loop, Worker=Worker, scheduler_kwargs={},
-                  worker_kwargs={}):
-    s = Scheduler(loop=loop, validate=True, **scheduler_kwargs)
-    done = s.start('127.0.0.1')
-    workers = [Worker(s.address, ncores=ncore[1], name=i,
+def start_cluster(ncores, scheduler_addr, loop, security=None,
+                  Worker=Worker, scheduler_kwargs={}, worker_kwargs={}):
+    s = Scheduler(loop=loop, validate=True, security=security,
+                  **scheduler_kwargs)
+    done = s.start(scheduler_addr)
+    workers = [Worker(s.address, ncores=ncore[1], name=i, security=security,
                       loop=loop, validate=True,
                       **(merge(worker_kwargs, ncore[2])
                          if len(ncore) > 2
                          else worker_kwargs))
-                for i, ncore in enumerate(ncores)]
+               for i, ncore in enumerate(ncores)]
     for w in workers:
         w.rpc = workers[0].rpc
 
@@ -479,17 +486,28 @@ def end_cluster(s, workers):
     def end_worker(w):
         with ignoring(TimeoutError, CommClosedError, EnvironmentError):
             yield w._close(report=False)
-        if w.local_dir and os.path.exists(w.local_dir):
-            shutil.rmtree(w.local_dir)
+        if isinstance(w, Nanny):
+            dir = w.worker_dir
+        else:
+            dir = w.local_dir
+        if dir and os.path.exists(dir):
+            shutil.rmtree(dir)
 
     yield [end_worker(w) for w in workers]
     yield s.close() # wait until scheduler stops completely
     s.stop()
 
 
-def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)], timeout=10,
-        Worker=Worker, client=False, scheduler_kwargs={}, worker_kwargs={},
-        active_rpc_timeout=0):
+def iscoroutinefunction(f):
+    if sys.version_info >= (3, 5) and inspect.iscoroutinefunction(f):
+        return True
+    return False
+
+
+def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
+                scheduler='127.0.0.1', timeout=10, security=None,
+                Worker=Worker, client=False, scheduler_kwargs={},
+                worker_kwargs={}, active_rpc_timeout=0):
     from distributed import Client
     """ Coroutine test with small cluster
 
@@ -502,25 +520,34 @@ def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)], timeout=10,
         end
     """
     def _(func):
-        cor = gen.coroutine(func)
+        cor = func
+        if not iscoroutinefunction(func):
+            cor = gen.coroutine(func)
 
         def test_func():
             with pristine_loop() as loop:
                 with check_active_rpc(loop, active_rpc_timeout):
-                    s, workers = loop.run_sync(lambda: start_cluster(ncores, loop,
-                                    Worker=Worker, scheduler_kwargs=scheduler_kwargs,
+                    s, workers = loop.run_sync(lambda: start_cluster(ncores,
+                                    scheduler, loop, security=security,
+                                    Worker=Worker,
+                                    scheduler_kwargs=scheduler_kwargs,
                                     worker_kwargs=worker_kwargs))
                     args = [s] + workers
 
                     if client:
-                        e = Client(s.address, loop=loop, start=False)
-                        loop.run_sync(e._start)
-                        args = [e] + args
+                        c = []
+                        @gen.coroutine
+                        def f():
+                            c2 = yield Client(s.address, loop=loop, security=security,
+                                              asynchronous=True)
+                            c.append(c2)
+                        loop.run_sync(f)
+                        args = c + args
                     try:
                         return loop.run_sync(lambda: cor(*args), timeout=timeout)
                     finally:
                         if client:
-                            loop.run_sync(e._shutdown)
+                            loop.run_sync(c[0]._shutdown)
                         loop.run_sync(lambda: end_cluster(s, workers))
 
                     for w in workers:
@@ -565,7 +592,9 @@ def terminate_process(proc):
             if sys.version_info[0] == 3:
                 proc.wait(10)
             else:
-                proc.wait()
+                start = time()
+                while proc.poll() is None and time() < start + 10:
+                    sleep(0.02)
         finally:
             # Make sure we don't leave the process lingering around
             with ignoring(OSError):
@@ -592,16 +621,13 @@ def popen(*args, **kwargs):
             terminate_process(proc)
         finally:
             # XXX Also dump stdout if return code != 0 ?
+            out, err = proc.communicate()
             if dump_stdout:
-                line = '\n\nPrint from stderr\n=================\n'
-                while line:
-                    print(line)
-                    line = proc.stderr.readline()
+                print('\n\nPrint from stderr\n=================\n')
+                print(err)
 
-                line = '\n\nPrint from stdout\n=================\n'
-                while line:
-                    print(line)
-                    line = proc.stdout.readline()
+                print('\n\nPrint from stdout\n=================\n')
+                print(out)
 
 
 def wait_for_port(address, timeout=5):
@@ -653,19 +679,20 @@ else:
 
 
 @gen.coroutine
-def assert_can_connect(addr, timeout=None):
+def assert_can_connect(addr, timeout=None, connection_args=None):
     """
     Check that it is possible to connect to the distributed *addr*
     within the given *timeout*.
     """
     if timeout is None:
         timeout = 0.2
-    comm = yield connect(addr, timeout=timeout)
+    comm = yield connect(addr, timeout=timeout,
+                         connection_args=connection_args)
     comm.abort()
 
 
 @gen.coroutine
-def assert_cannot_connect(addr, timeout=None):
+def assert_cannot_connect(addr, timeout=None, connection_args=None):
     """
     Check that it is impossible to connect to the distributed *addr*
     within the given *timeout*.
@@ -673,89 +700,95 @@ def assert_cannot_connect(addr, timeout=None):
     if timeout is None:
         timeout = 0.2
     with pytest.raises(EnvironmentError):
-        comm = yield connect(addr, timeout=timeout)
+        comm = yield connect(addr, timeout=timeout,
+                             connection_args=connection_args)
         comm.abort()
 
 
 @gen.coroutine
-def assert_can_connect_from_everywhere_4_6(port, timeout=None):
+def assert_can_connect_from_everywhere_4_6(port, timeout=None, connection_args=None):
     """
     Check that the local *port* is reachable from all IPv4 and IPv6 addresses.
     """
+    args = (timeout, connection_args)
     futures = [
-        assert_can_connect('tcp://127.0.0.1:%d' % port, timeout),
-        assert_can_connect('tcp://%s:%d' % (get_ip(), port), timeout),
+        assert_can_connect('tcp://127.0.0.1:%d' % port, *args),
+        assert_can_connect('tcp://%s:%d' % (get_ip(), port), *args),
         ]
     if has_ipv6():
         futures += [
-            assert_can_connect('tcp://[::1]:%d' % port, timeout),
-            assert_can_connect('tcp://[%s]:%d' % (get_ipv6(), port), timeout),
+            assert_can_connect('tcp://[::1]:%d' % port, *args),
+            assert_can_connect('tcp://[%s]:%d' % (get_ipv6(), port), *args),
             ]
     yield futures
 
 @gen.coroutine
-def assert_can_connect_from_everywhere_4(port, timeout=None):
+def assert_can_connect_from_everywhere_4(port, timeout=None, connection_args=None):
     """
     Check that the local *port* is reachable from all IPv4 addresses.
     """
+    args = (timeout, connection_args)
     futures = [
-            assert_can_connect('tcp://127.0.0.1:%d' % port, timeout),
-            assert_can_connect('tcp://%s:%d' % (get_ip(), port), timeout),
+            assert_can_connect('tcp://127.0.0.1:%d' % port, *args),
+            assert_can_connect('tcp://%s:%d' % (get_ip(), port), *args),
         ]
     if has_ipv6():
         futures += [
-            assert_cannot_connect('tcp://[::1]:%d' % port, timeout),
-            assert_cannot_connect('tcp://[%s]:%d' % (get_ipv6(), port), timeout),
+            assert_cannot_connect('tcp://[::1]:%d' % port, *args),
+            assert_cannot_connect('tcp://[%s]:%d' % (get_ipv6(), port), *args),
             ]
     yield futures
 
 @gen.coroutine
-def assert_can_connect_locally_4(port, timeout=None):
+def assert_can_connect_locally_4(port, timeout=None, connection_args=None):
     """
     Check that the local *port* is only reachable from local IPv4 addresses.
     """
+    args = (timeout, connection_args)
     futures = [
-        assert_can_connect('tcp://127.0.0.1:%d' % port, timeout),
+        assert_can_connect('tcp://127.0.0.1:%d' % port, *args),
         ]
     if get_ip() != '127.0.0.1':  # No outside IPv4 connectivity?
         futures += [
-            assert_cannot_connect('tcp://%s:%d' % (get_ip(), port), timeout),
+            assert_cannot_connect('tcp://%s:%d' % (get_ip(), port), *args),
             ]
     if has_ipv6():
         futures += [
-            assert_cannot_connect('tcp://[::1]:%d' % port, timeout),
-            assert_cannot_connect('tcp://[%s]:%d' % (get_ipv6(), port), timeout),
+            assert_cannot_connect('tcp://[::1]:%d' % port, *args),
+            assert_cannot_connect('tcp://[%s]:%d' % (get_ipv6(), port), *args),
             ]
     yield futures
 
 @gen.coroutine
-def assert_can_connect_from_everywhere_6(port, timeout=None):
+def assert_can_connect_from_everywhere_6(port, timeout=None, connection_args=None):
     """
     Check that the local *port* is reachable from all IPv6 addresses.
     """
     assert has_ipv6()
+    args = (timeout, connection_args)
     futures = [
-        assert_cannot_connect('tcp://127.0.0.1:%d' % port, timeout),
-        assert_cannot_connect('tcp://%s:%d' % (get_ip(), port), timeout),
-        assert_can_connect('tcp://[::1]:%d' % port, timeout),
-        assert_can_connect('tcp://[%s]:%d' % (get_ipv6(), port), timeout),
+        assert_cannot_connect('tcp://127.0.0.1:%d' % port, *args),
+        assert_cannot_connect('tcp://%s:%d' % (get_ip(), port), *args),
+        assert_can_connect('tcp://[::1]:%d' % port, *args),
+        assert_can_connect('tcp://[%s]:%d' % (get_ipv6(), port), *args),
         ]
     yield futures
 
 @gen.coroutine
-def assert_can_connect_locally_6(port, timeout=None):
+def assert_can_connect_locally_6(port, timeout=None, connection_args=None):
     """
     Check that the local *port* is only reachable from local IPv6 addresses.
     """
     assert has_ipv6()
+    args = (timeout, connection_args)
     futures = [
-        assert_cannot_connect('tcp://127.0.0.1:%d' % port, timeout),
-        assert_cannot_connect('tcp://%s:%d' % (get_ip(), port), timeout),
-        assert_can_connect('tcp://[::1]:%d' % port, timeout),
+        assert_cannot_connect('tcp://127.0.0.1:%d' % port, *args),
+        assert_cannot_connect('tcp://%s:%d' % (get_ip(), port), *args),
+        assert_can_connect('tcp://[::1]:%d' % port, *args),
         ]
     if get_ipv6() != '::1':  # No outside IPv6 connectivity?
         futures += [
-            assert_cannot_connect('tcp://[%s]:%d' % (get_ipv6(), port), timeout),
+            assert_cannot_connect('tcp://[%s]:%d' % (get_ipv6(), port), *args),
             ]
     yield futures
 
@@ -784,3 +817,107 @@ def captured_handler(handler):
         yield handler.stream
     finally:
         handler.stream = orig_stream
+
+
+@contextmanager
+def new_config(new_config):
+    """
+    Temporarily change configuration dictionary.
+    """
+    orig_config = config.copy()
+    try:
+        config.clear()
+        config.update(new_config)
+        yield
+    finally:
+        config.clear()
+        config.update(orig_config)
+
+
+@contextmanager
+def new_config_file(c):
+    """
+    Temporarily change configuration file to match dictionary *c*.
+    """
+    import yaml
+    old_file = os.environ.get('DASK_CONFIG')
+    fd, path = tempfile.mkstemp(prefix='dask-config')
+    try:
+        with os.fdopen(fd, 'w') as f:
+            f.write(yaml.dump(c))
+        os.environ['DASK_CONFIG'] = path
+        try:
+            yield
+        finally:
+            if old_file:
+                os.environ['DASK_CONFIG'] = old_file
+            else:
+                del os.environ['DASK_CONFIG']
+    finally:
+        os.remove(path)
+
+
+certs_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                         'tests'))
+
+def get_cert(filename):
+    """
+    Get the path to one of the test TLS certificates.
+    """
+    path = os.path.join(certs_dir, filename)
+    assert os.path.exists(path), path
+    return path
+
+
+def tls_config():
+    """
+    A functional TLS configuration with our test certs.
+    """
+    ca_file = get_cert('tls-ca-cert.pem')
+    keycert = get_cert('tls-key-cert.pem')
+
+    c = {
+        'tls': {
+            'ca-file': ca_file,
+            'client': {
+                'cert': keycert,
+                },
+            'scheduler': {
+                'cert': keycert,
+                },
+            'worker': {
+                'cert': keycert,
+                },
+            },
+        }
+    return c
+
+
+def tls_only_config():
+    """
+    A functional TLS configuration with our test certs, disallowing
+    plain TCP communications.
+    """
+    c = tls_config()
+    c['require-encryption'] = True
+    return c
+
+
+def tls_security():
+    """
+    A Security object with proper TLS configuration.
+    """
+    with new_config(tls_config()):
+        sec = Security()
+    return sec
+
+
+def tls_only_security():
+    """
+    A Security object with proper TLS configuration and disallowing plain
+    TCP communications.
+    """
+    with new_config(tls_only_config()):
+        sec = Security()
+    assert sec.require_encryption
+    return sec
