@@ -28,15 +28,18 @@ from .comm import get_address_host, get_local_address_for
 from .config import config
 from .compatibility import unicode
 from .core import (error_message, CommClosedError,
-                   rpc, Server, pingpong, coerce_to_address)
+                   rpc, pingpong, coerce_to_address)
 from .metrics import time
+from .node import ServerNode
 from .preloading import preload_modules
 from .protocol import (pickle, to_serialize, deserialize_bytes,
                        serialize_bytelist)
-from .sizeof import sizeof
+from .security import Security
+from .sizeof import safe_sizeof as sizeof
 from .threadpoolexecutor import ThreadPoolExecutor
 from .utils import (funcname, get_ip, has_arg, _maybe_complex, log_errors,
-                    ignoring, validate_key, mp_context, import_file)
+                    ignoring, validate_key, mp_context, import_file,
+                    silence_logging)
 from .utils_comm import pack_data, gather_from_workers
 
 _ncores = mp_context.cpu_count()
@@ -53,7 +56,7 @@ try:
     import psutil
     TOTAL_MEMORY = psutil.virtual_memory().total
 except ImportError:
-    logger.warn("Please install psutil to estimate worker memory use")
+    logger.warning("Please install psutil to estimate worker memory use")
     TOTAL_MEMORY = 8e9
     psutil = None
 
@@ -64,27 +67,36 @@ PROCESSING = ('waiting', 'ready', 'constrained', 'executing', 'long-running')
 READY = ('ready', 'constrained')
 
 
-class WorkerBase(Server):
+class WorkerBase(ServerNode):
     def __init__(self, scheduler_ip, scheduler_port=None, ncores=None,
                  loop=None, local_dir=None, services=None, service_ports=None,
                  name=None, heartbeat_interval=5000, reconnect=True,
                  memory_limit='auto', executor=None, resources=None,
-                 silence_logs=None, death_timeout=None, preload=(), **kwargs):
+                 silence_logs=None, death_timeout=None, preload=(),
+                 security=None, **kwargs):
         if scheduler_port is None:
             scheduler_addr = coerce_to_address(scheduler_ip)
         else:
             scheduler_addr = coerce_to_address((scheduler_ip, scheduler_port))
         self._port = 0
         self.ncores = ncores or _ncores
-        self.local_dir = local_dir or tempfile.mkdtemp(prefix='worker-')
         self.total_resources = resources or {}
         self.available_resources = (resources or {}).copy()
         self.death_timeout = death_timeout
         self.preload = preload
         if silence_logs:
-            logger.setLevel(silence_logs)
-        if not os.path.exists(self.local_dir):
-            os.mkdir(self.local_dir)
+            silence_logging(level=silence_logs)
+
+        if local_dir:
+            local_dir = os.path.abspath(local_dir)
+            if not os.path.exists(local_dir):
+                os.mkdir(local_dir)
+        self.local_dir = tempfile.mkdtemp(prefix='worker-', dir=local_dir)
+
+        self.security = security or Security()
+        assert isinstance(self.security, Security)
+        self.connection_args = self.security.get_connection_args('worker')
+        self.listen_args = self.security.get_listen_args('worker')
 
         if memory_limit == 'auto':
             memory_limit = int(TOTAL_MEMORY * 0.6 * min(1, self.ncores / _ncores))
@@ -109,7 +121,7 @@ class WorkerBase(Server):
         self._closed = Event()
         self.reconnect = reconnect
         self.executor = executor or ThreadPoolExecutor(self.ncores)
-        self.scheduler = rpc(scheduler_addr)
+        self.scheduler = rpc(scheduler_addr, connection_args=self.connection_args)
         self.name = name
         self.heartbeat_interval = heartbeat_interval
         self.heartbeat_active = False
@@ -143,7 +155,9 @@ class WorkerBase(Server):
           'keys': self.keys,
         }
 
-        super(WorkerBase, self).__init__(handlers, io_loop=self.loop, **kwargs)
+        super(WorkerBase, self).__init__(handlers, io_loop=self.loop,
+                                         connection_args=self.connection_args,
+                                         **kwargs)
 
         self.heartbeat_callback = PeriodicCallback(self.heartbeat,
                                                    self.heartbeat_interval,
@@ -232,7 +246,12 @@ class WorkerBase(Server):
             else:
                 port = 0
 
-            self.services[k] = v(self, io_loop=self.loop)
+            if isinstance(v, tuple):
+                v, kwargs = v
+            else:
+                v, kwargs = v, {}
+
+            self.services[k] = v(self, io_loop=self.loop, **kwargs)
             self.services[k].listen((listen_ip, port))
             self.service_ports[k] = self.services[k].port
 
@@ -243,16 +262,18 @@ class WorkerBase(Server):
         # XXX Factor this out
         if not addr_or_port:
             # Default address is the required one to reach the scheduler
-            self.listen(get_local_address_for(self.scheduler.address))
+            self.listen(get_local_address_for(self.scheduler.address),
+                        listen_args=self.listen_args)
             self.ip = get_address_host(self.address)
         elif isinstance(addr_or_port, int):
             # addr_or_port is an integer => assume TCP
             self.ip = get_ip(
                 get_address_host(self.scheduler.address)
             )
-            self.listen((self.ip, addr_or_port))
+            self.listen((self.ip, addr_or_port),
+                        listen_args=self.listen_args)
         else:
-            self.listen(addr_or_port)
+            self.listen(addr_or_port, listen_args=self.listen_args)
             self.ip = get_address_host(self.address)
 
         self.name = self.name or self.address
@@ -399,8 +420,9 @@ class WorkerBase(Server):
             logger.debug("Deleted %d keys", len(keys))
             if report:
                 logger.debug("Reporting loss of keys to scheduler")
+                # TODO: this route seems to not exist?
                 yield self.scheduler.remove_keys(address=self.address,
-                                              keys=list(keys))
+                                                 keys=list(keys))
         raise Return('OK')
 
     @gen.coroutine
@@ -529,10 +551,10 @@ class WorkerBase(Server):
                     for k, v in who_has.items()
                     if k not in self.data}
         result, missing_keys, missing_workers = yield gather_from_workers(
-                who_has)
+                who_has, rpc=self.rpc)
         if missing_keys:
-            logger.warn("Could not find data: %s on workers: %s",
-                        missing_keys, missing_workers)
+            logger.warning("Could not find data: %s on workers: %s (who_has: %s)",
+                           missing_keys, missing_workers, who_has)
             raise Return({'status': 'missing-data',
                           'keys': missing_keys})
         else:
@@ -733,7 +755,7 @@ def run(server, comm, function, args=(), kwargs={}, is_coro=False, wait=True):
         if is_coro:
             result = (yield result) if wait else None
     except Exception as e:
-        logger.warn(" Run Failed\n"
+        logger.warning(" Run Failed\n"
             "Function: %s\n"
             "args:     %s\n"
             "kwargs:   %s\n",
@@ -959,7 +981,8 @@ class Worker(WorkerBase):
 
         self._transitions = {
                 ('waiting', 'ready'): self.transition_waiting_ready,
-                ('waiting', 'memory'): self.transition_waiting_memory,
+                ('waiting', 'memory'): self.transition_waiting_done,
+                ('waiting', 'error'): self.transition_waiting_done,
                 ('ready', 'executing'): self.transition_ready_executing,
                 ('ready', 'memory'): self.transition_ready_memory,
                 ('constrained', 'executing'): self.transition_constrained_executing,
@@ -974,7 +997,7 @@ class Worker(WorkerBase):
                 ('waiting', 'flight'): self.transition_dep_waiting_flight,
                 ('waiting', 'memory'): self.transition_dep_waiting_memory,
                 ('flight', 'waiting'): self.transition_dep_flight_waiting,
-                ('flight', 'memory'): self.transition_dep_flight_memory
+                ('flight', 'memory'): self.transition_dep_flight_memory,
         }
 
         self.incoming_transfer_log = deque(maxlen=(100000))
@@ -1036,7 +1059,7 @@ class Worker(WorkerBase):
                         self.add_task(**msg)
                     elif op == 'release-task':
                         self.log.append((msg['key'], 'release-task'))
-                        self.release_key(**msg)
+                        self.release_key(report=False, **msg)
                     elif op == 'delete-data':
                         self.delete_data(**msg)
                     else:
@@ -1093,7 +1116,7 @@ class Worker(WorkerBase):
                 if stop - start > 0.010:
                     self.startstops[key].append(('deserialize', start, stop))
             except Exception as e:
-                logger.warn("Could not deserialize task", exc_info=True)
+                logger.warning("Could not deserialize task", exc_info=True)
                 emsg = error_message(e)
                 emsg['key'] = key
                 emsg['op'] = 'task-erred'
@@ -1203,7 +1226,7 @@ class Worker(WorkerBase):
             except KeyError:
                 pass
 
-            if not self.who_has[dep]:
+            if not self.who_has.get(dep):
                 if dep not in self._missing_dep_flight:
                     self._missing_dep_flight.add(dep)
                     self.loop.add_callback(self.handle_missing_dep, dep)
@@ -1283,7 +1306,7 @@ class Worker(WorkerBase):
                 import pdb; pdb.set_trace()
             raise
 
-    def transition_waiting_memory(self, key, value=None):
+    def transition_waiting_done(self, key, value=None):
         try:
             if self.validate:
                 assert self.task_state[key] == 'waiting'
@@ -1334,6 +1357,7 @@ class Worker(WorkerBase):
                 assert key not in self.waiting_for_data
                 assert key not in self.ready
 
+            out = None
             if key in self.resource_restrictions:
                 for resource, quantity in self.resource_restrictions[key].items():
                     self.available_resources[resource] += quantity
@@ -1345,8 +1369,17 @@ class Worker(WorkerBase):
                 self.long_running.remove(key)
 
             if value is not no_value:
-                self.task_state[key] = 'memory'
-                self.put_key_in_memory(key, value)
+                try:
+                    self.task_state[key] = 'memory'
+                    self.put_key_in_memory(key, value, transition=False)
+                except Exception as e:
+                    logger.info("Failed to put key in memory", exc_info=True)
+                    msg = error_message(e)
+                    self.exceptions[key] = msg['exception']
+                    self.tracebacks[key] = msg['traceback']
+                    self.task_state[key] = 'error'
+                    out = 'error'
+
                 if key in self.dep_state:
                     self.transition_dep(key, 'memory')
 
@@ -1355,6 +1388,8 @@ class Worker(WorkerBase):
             else:
                 raise CommClosedError
 
+            return out
+
         except EnvironmentError:
             logger.info("Comm closed")
         except Exception as e:
@@ -1490,7 +1525,7 @@ class Worker(WorkerBase):
             d['startstops'] = self.startstops[key]
         self.batched_stream.send(d)
 
-    def put_key_in_memory(self, key, value):
+    def put_key_in_memory(self, key, value, transition=True):
         if key in self.data:
             return
 
@@ -1515,7 +1550,7 @@ class Worker(WorkerBase):
                 if not self.waiting_for_data[dep]:
                     self.transition(dep, 'ready')
 
-        if key in self.task_state:
+        if transition and key in self.task_state:
             self.transition(key, 'memory')
 
         self.log.append((key, 'put-in-memory'))
@@ -1581,9 +1616,14 @@ class Worker(WorkerBase):
                     self.batched_stream.send({'op': 'add-keys',
                                               'keys': list(response)})
             except EnvironmentError as e:
-                logger.error("Worker stream died during communication: %s",
-                             worker)
+                logger.exception("Worker stream died during communication: %s",
+                                 worker)
                 self.log.append(('receive-dep-failed', worker))
+                for d in self.has_what.pop(worker):
+                    self.who_has[d].remove(worker)
+                    if not self.who_has[d]:
+                        del self.who_has[d]
+
             except Exception as e:
                 logger.exception(e)
                 if self.batched_stream and LOG_PDB:
@@ -1607,6 +1647,15 @@ class Worker(WorkerBase):
                 self.ensure_computing()
                 self.ensure_communicating()
 
+    def bad_dep(self, dep):
+        exc = ValueError("Could not find dependent %s.  Check worker logs" % str(dep))
+        for key in self.dependents[dep]:
+            msg = error_message(exc)
+            self.exceptions[key] = msg['exception']
+            self.tracebacks[key] = msg['traceback']
+            self.transition(key, 'error')
+        self.release_dep(dep)
+
     @gen.coroutine
     def handle_missing_dep(self, *deps):
         original_deps = list(deps)
@@ -1620,7 +1669,7 @@ class Worker(WorkerBase):
                 suspicious = self.suspicious_deps[dep]
                 if suspicious > 5:
                     deps.remove(dep)
-                    self.release_dep(dep)
+                    self.bad_dep(dep)
             if not deps:
                 return
 
@@ -1629,11 +1678,12 @@ class Worker(WorkerBase):
                             dep, self.suspicious_deps[dep])
 
             who_has = yield self.scheduler.who_has(keys=list(deps))
+            who_has = {k: v for k, v in who_has.items() if v}
             self.update_who_has(who_has)
             for dep in deps:
                 self.suspicious_deps[dep] += 1
 
-                if dep not in who_has:
+                if not who_has.get(dep):
                     self.log.append((dep, 'no workers found',
                                      self.dependents.get(dep)))
                     self.release_dep(dep)
@@ -1661,6 +1711,8 @@ class Worker(WorkerBase):
     def update_who_has(self, who_has):
         try:
             for dep, workers in who_has.items():
+                if not workers:
+                    continue
                 if dep in self.who_has:
                     self.who_has[dep].update(workers)
                 else:
@@ -1674,7 +1726,7 @@ class Worker(WorkerBase):
                 import pdb; pdb.set_trace()
             raise
 
-    def release_key(self, key, cause=None, reason=None):
+    def release_key(self, key, cause=None, reason=None, report=True):
         try:
             if key not in self.task_state:
                 return
@@ -1719,7 +1771,7 @@ class Worker(WorkerBase):
             if key in self.resource_restrictions:
                 del self.resource_restrictions[key]
 
-            if state in PROCESSING:  # not finished
+            if report and state in PROCESSING:  # not finished
                 self.batched_stream.send({'op': 'release',
                                           'key': key,
                                           'cause': cause})
@@ -1731,16 +1783,20 @@ class Worker(WorkerBase):
                 import pdb; pdb.set_trace()
             raise
 
-    def release_dep(self, dep):
+    def release_dep(self, dep, report=False):
         try:
             if dep not in self.dep_state:
                 return
             self.log.append((dep, 'release-dep'))
-            self.dep_state.pop(dep)
+            state = self.dep_state.pop(dep)
 
             if dep in self.suspicious_deps:
                 del self.suspicious_deps[dep]
 
+            if dep in self.who_has:
+                for worker in self.who_has.pop(dep):
+                    self.has_what[worker].remove(dep)
+
             if dep not in self.task_state:
                 if dep in self.data:
                     del self.data[dep]
@@ -1754,6 +1810,10 @@ class Worker(WorkerBase):
                 self.dependencies[key].remove(dep)
                 if self.task_state[key] != 'memory':
                     self.release_key(key, cause=dep)
+
+            if report and state == 'memory':
+                self.batched_stream.send({'op': 'release-worker-data',
+                                          'keys': [dep]})
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
@@ -1871,7 +1931,7 @@ class Worker(WorkerBase):
             else:
                 self.exceptions[key] = result['exception']
                 self.tracebacks[key] = result['traceback']
-                logger.warn(" Compute Failed\n"
+                logger.warning(" Compute Failed\n"
                     "Function:  %s\n"
                     "args:      %s\n"
                     "kwargs:    %s\n"
